@@ -103,7 +103,7 @@ prepare_app(int argc, char *argv[])
   button = gtk_button_new_with_label (_("Hello GNOME"));
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
      		      GTK_SIGNAL_FUNC (hello_cb), NULL);
-  gtk_container_border_width (GTK_CONTAINER (button), 60);
+  gtk_container_set_border_width (GTK_CONTAINER (button), 60);
   gnome_app_set_contents ( GNOME_APP (app), button);
 
   /* We now show the widgets, the order doesn't matter, but i suggests 
