@@ -132,6 +132,43 @@ static TProfile *Base = 0;
    static ParsedPath *parse_path (const char *path, gint priv); */
 #include "parse-path.cP"
 
+static void 
+free_keys (TKeys *p)
+{
+	if (!p)
+		return;
+	free_keys (p->link);
+	g_free (p->key_name);
+	g_free (p->value);
+	g_free (p);
+}
+
+static void 
+free_sections (TSecHeader *p)
+{
+	if (!p)
+		return;
+	free_sections (p->link);
+	free_keys (p->keys);
+	g_free (p->section_name);
+	p->link = 0;
+	p->keys = 0;
+	g_free (p);
+}
+
+static void 
+free_profile (TProfile *p)
+{
+	if (!p)
+		return;
+	if(Current == p)
+		Current = NULL;
+	free_profile (p->link);
+	free_sections (p->section);
+	g_free (p->filename);
+	g_free (p);
+}
+
 static int 
 is_loaded (const char *filename, TSecHeader **section)
 {
@@ -147,9 +184,16 @@ is_loaded (const char *filename, TSecHeader **section)
 		if (Current->last_checked != time (NULL)){
 			if (stat (filename, &st) == -1)
 				st.st_mtime = 0;
-			Current->last_checked = time (NULL);
-			if (Current->mtime != st.st_mtime)
+			if (Current->mtime != st.st_mtime) {
+				Current = NULL;
+				free_sections (Current->section);
+				Current->section = NULL;
+				Current->filename[0] = '\0';
+				Current->written_to = TRUE;
+				Current->to_be_deleted = FALSE;
 				return 0;
+			}
+			Current->last_checked = time (NULL);
 		}
 		*section = Current->section;
 		return 1;
@@ -170,9 +214,17 @@ is_loaded (const char *filename, TSecHeader **section)
 			if (p->last_checked != time (NULL)){
 				if (stat (filename, &st) == -1)
 					st.st_mtime = 0;
-				p->last_checked = time (NULL);
-				if (p->mtime != st.st_mtime)
+				if (p->mtime != st.st_mtime) {
+					if(p == Current)
+						Current = NULL;
+					free_sections (p->section);
+					p->section = NULL;
+					p->filename[0] = '\0';
+					p->written_to = TRUE;
+					p->to_be_deleted = FALSE;
 					return 0;
+				}
+				p->last_checked = time (NULL);
 			}
 			Current = p;
 			*section = p->section;
@@ -338,8 +390,6 @@ load (const char *file)
 				state = c == '\n' ? KeyDef : IgnoreToEOL;
 				next = CharBuffer;
 #ifdef GNOME_ENABLE_DEBUG
-				printf ("[%s] (%s)=%s\n", SecHeader->section_name,
-					SecHeader->keys->key_name, SecHeader->keys->value);
 #endif
 			} else
 				*next++ = c;
@@ -540,42 +590,6 @@ check_path(char *path, mode_t newmode)
 	return TRUE;
 }
 
-static void 
-free_keys (TKeys *p)
-{
-	if (!p)
-		return;
-	free_keys (p->link);
-	g_free (p->key_name);
-	g_free (p->value);
-	g_free (p);
-}
-
-static void 
-free_sections (TSecHeader *p)
-{
-	if (!p)
-		return;
-	free_sections (p->link);
-	free_keys (p->keys);
-	g_free (p->section_name);
-	p->link = 0;
-	p->keys = 0;
-	g_free (p);
-}
-
-static void 
-free_profile (TProfile *p)
-{
-	if (!p)
-		return;
-	if(Current == p)
-		Current = NULL;
-	free_profile (p->link);
-	free_sections (p->section);
-	g_free (p->filename);
-	g_free (p);
-}
 
 
 static void 
@@ -592,13 +606,12 @@ dump_profile (TProfile *p, int one_only)
 	 * was this profile written to?, if not it's not necessary to dump
 	 * it to disk
 	 */
-	if (!p->to_be_deleted)
-		if(!p->written_to)
-			return;
+	if (!p->to_be_deleted && !p->written_to)
+		return;
 	
 
 	/* .ado: p->filename can be empty, it's better to jump over */
-	if (p->filename[0] != (char) 0) {
+	if (p->filename[0] != '\0') {
 
 		/*
 		 * this file was added to after it was cleaned so it doesn't
@@ -612,7 +625,7 @@ dump_profile (TProfile *p, int one_only)
 			unlink(p->filename);
 			/* this already must have been true */
 			/*p->section = 0;*/
-			p->filename [0] = 0;
+			p->filename [0] = '\0';
 			p->written_to = TRUE;
 			p->to_be_deleted = FALSE;
 			if(p==Current)
@@ -731,7 +744,7 @@ _gnome_config_clean_file (const char *path, gint priv)
 			continue;
 		
 		free_sections (p->section);
-		p->section = 0;
+		p->section = NULL;
 		p->written_to = TRUE;
 		p->to_be_deleted = TRUE;
 		release_path (pp);
@@ -1166,7 +1179,8 @@ void
 gnome_config_drop_all (void)
 {
 	free_profile (Base);
-	Base = 0;
+	Base = NULL;
+	Current = NULL;
 }
 
 /**
@@ -1211,8 +1225,10 @@ _gnome_config_get_int_with_default (const char *path, gboolean *def, gint priv)
 			   def);
 
 	/* It isn't an error if the key is not found.  */
-	if (r == NULL)
+	if (r == NULL) {
+		release_path (pp);
 		return 0;
+	}
 
 	v = atoi (r);
 	release_path (pp);
@@ -1262,8 +1278,10 @@ _gnome_config_get_float_with_default (const char *path, gboolean *def, gint priv
 			   def);
 
 	/* It isn't an error if the key is not found.  */
-	if (r == NULL)
+	if (r == NULL) {
+		release_path (pp);
 		return 0;
+	}
 
 	v = strtod(r, NULL);
 	release_path (pp);
